@@ -1,32 +1,30 @@
+use crate::cryptosystems::AsymmetricCryptosystem;
 use crate::number_theory::gen_safe_prime;
 use crate::randomness::SecureRng;
-use crate::{AsymmetricCryptosystem, Enrichable, RichCiphertext};
+use crate::{BitsOfSecurity, Enrichable, RichCiphertext};
 use rug::Integer;
 use std::ops::{Mul, Rem};
 
-/// Multiplicatively homomorphic ElGamal over a safe prime group.
+/// Multiplicatively homomorphic ElGamal over a safe prime group where the generator is 4.
 ///
 /// As an example we compute the product between 4 and 6 using ElGamal's homomorphic property.
 /// ```
 /// # use scicrypt::cryptosystems::integer_el_gamal::IntegerElGamal;
 /// # use rand_core::OsRng;
 /// # use scicrypt::randomness::SecureRng;
-/// # use scicrypt::{AsymmetricCryptosystem, Enrichable};
 /// # use rug::Integer;
+/// # use scicrypt::cryptosystems::AsymmetricCryptosystem;
+/// # use scicrypt::{Enrichable, BitsOfSecurity};
 /// let mut rng = SecureRng::new(OsRng);
-/// let el_gamal = IntegerElGamal::new(128, &mut rng);
-/// let (public_key, secret_key) = el_gamal.generate_keys(&mut rng);
+/// let (public_key, secret_key) = IntegerElGamal::generate_keys(&BitsOfSecurity::Other {pk_bits: 160}, &mut rng);
 ///
-/// let rich_ciphertext_1 = el_gamal.encrypt(&Integer::from(4), &public_key, &mut rng).enrich(&public_key);
-/// let rich_ciphertext_2 = el_gamal.encrypt(&Integer::from(6), &public_key, &mut rng).enrich(&public_key);
+/// let rich_ciphertext_1 = IntegerElGamal::encrypt(&Integer::from(4), &public_key, &mut rng).enrich(&public_key);
+/// let rich_ciphertext_2 = IntegerElGamal::encrypt(&Integer::from(6), &public_key, &mut rng).enrich(&public_key);
 ///
-/// println!("[4] * [6] = [{}]", el_gamal.decrypt(&(&rich_ciphertext_1 * &rich_ciphertext_2), &secret_key));
+/// println!("[4] * [6] = [{}]", IntegerElGamal::decrypt(&(&rich_ciphertext_1 * &rich_ciphertext_2), &secret_key));
 /// // Prints: "[4] * [6] = [24]".
 /// ```
-pub struct IntegerElGamal {
-    modulus: Integer,
-    generator: Integer,
-}
+pub struct IntegerElGamal;
 
 /// Public key containing the ElGamal encryption key and the modulus of the group.
 pub struct IntegerElGamalPublicKey {
@@ -40,29 +38,6 @@ pub struct IntegerElGamalCiphertext {
     pub(crate) c2: Integer,
 }
 
-impl IntegerElGamal {
-    /// Creates a fresh `IntegerElGamal` instance over a randomly chosen safe prime group of size
-    /// `group_size`.
-    /// ```
-    /// # use scicrypt::cryptosystems::integer_el_gamal::IntegerElGamal;
-    /// # use rand_core::OsRng;
-    /// # use scicrypt::randomness::SecureRng;
-    /// let mut rng = SecureRng::new(OsRng);
-    /// let el_gamal = IntegerElGamal::new(512, &mut rng);
-    /// ```
-    pub fn new<R: rand_core::RngCore + rand_core::CryptoRng>(
-        group_size: u32,
-        rng: &mut SecureRng<R>,
-    ) -> Self {
-        let modulus = gen_safe_prime(group_size, rng);
-
-        IntegerElGamal {
-            modulus,
-            generator: Integer::from(4),
-        }
-    }
-}
-
 impl Enrichable<IntegerElGamalPublicKey> for IntegerElGamalCiphertext {}
 
 impl AsymmetricCryptosystem for IntegerElGamal {
@@ -76,26 +51,25 @@ impl AsymmetricCryptosystem for IntegerElGamal {
     /// # use scicrypt::cryptosystems::integer_el_gamal::IntegerElGamal;
     /// # use rand_core::OsRng;
     /// # use scicrypt::randomness::SecureRng;
-    /// # use scicrypt::AsymmetricCryptosystem;
+    /// # use scicrypt::cryptosystems::AsymmetricCryptosystem;
+    /// # use scicrypt::BitsOfSecurity;
     /// # let mut rng = SecureRng::new(OsRng);
-    /// # let el_gamal = IntegerElGamal::new(128, &mut rng);
-    /// let (public_key, secret_key) = el_gamal.generate_keys(&mut rng);
+    /// let (public_key, secret_key) = IntegerElGamal::generate_keys(&BitsOfSecurity::Other {pk_bits: 160}, &mut rng);
     /// ```
     fn generate_keys<R: rand_core::RngCore + rand_core::CryptoRng>(
-        &self,
+        security_param: &BitsOfSecurity,
         rng: &mut SecureRng<R>,
     ) -> (Self::PublicKey, Self::SecretKey) {
-        let q = Integer::from(&self.modulus >> 1);
+        let modulus = gen_safe_prime(security_param.to_public_key_bit_length(), rng);
+
+        let q = Integer::from(&modulus >> 1);
         let secret_key = q.random_below(&mut rng.rug_rng());
-        let public_key = Integer::from(
-            self.generator
-                .secure_pow_mod_ref(&secret_key, &self.modulus),
-        );
+        let public_key = Integer::from(Integer::from(4).secure_pow_mod_ref(&secret_key, &modulus));
 
         (
             IntegerElGamalPublicKey {
                 h: public_key,
-                modulus: Integer::from(&self.modulus),
+                modulus,
             },
             secret_key,
         )
@@ -106,15 +80,14 @@ impl AsymmetricCryptosystem for IntegerElGamal {
     /// # use scicrypt::cryptosystems::integer_el_gamal::IntegerElGamal;
     /// # use rand_core::OsRng;
     /// # use scicrypt::randomness::SecureRng;
-    /// # use scicrypt::AsymmetricCryptosystem;
     /// # use rug::Integer;
+    /// # use scicrypt::cryptosystems::AsymmetricCryptosystem;
+    /// # use scicrypt::BitsOfSecurity;
     /// # let mut rng = SecureRng::new(OsRng);
-    /// # let el_gamal = IntegerElGamal::new(128, &mut rng);
-    /// # let (public_key, secret_key) = el_gamal.generate_keys(&mut rng);
-    /// let ciphertext = el_gamal.encrypt(&Integer::from(5), &public_key, &mut rng);
+    /// # let (public_key, secret_key) = IntegerElGamal::generate_keys(&BitsOfSecurity::Other {pk_bits: 160}, &mut rng);
+    /// let ciphertext = IntegerElGamal::encrypt(&Integer::from(5), &public_key, &mut rng);
     /// ```
     fn encrypt<R: rand_core::RngCore + rand_core::CryptoRng>(
-        &self,
         plaintext: &Self::Plaintext,
         public_key: &Self::PublicKey,
         rng: &mut SecureRng<R>,
@@ -123,7 +96,7 @@ impl AsymmetricCryptosystem for IntegerElGamal {
         let y = q.random_below(&mut rng.rug_rng());
 
         IntegerElGamalCiphertext {
-            c1: Integer::from(self.generator.secure_pow_mod_ref(&y, &public_key.modulus)),
+            c1: Integer::from(Integer::from(4).secure_pow_mod_ref(&y, &public_key.modulus)),
             c2: (plaintext
                 * Integer::from(public_key.h.secure_pow_mod_ref(&y, &public_key.modulus)))
             .rem(&public_key.modulus),
@@ -135,18 +108,17 @@ impl AsymmetricCryptosystem for IntegerElGamal {
     /// # use scicrypt::cryptosystems::integer_el_gamal::IntegerElGamal;
     /// # use rand_core::OsRng;
     /// # use scicrypt::randomness::SecureRng;
-    /// # use scicrypt::{AsymmetricCryptosystem, Enrichable};
     /// # use rug::Integer;
+    /// # use scicrypt::cryptosystems::AsymmetricCryptosystem;
+    /// # use scicrypt::{Enrichable, BitsOfSecurity};
     /// # let mut rng = SecureRng::new(OsRng);
-    /// # let el_gamal = IntegerElGamal::new(128, &mut rng);
-    /// # let (public_key, secret_key) = el_gamal.generate_keys(&mut rng);
-    /// # let ciphertext = el_gamal.encrypt(&Integer::from(5), &public_key, &mut rng);
+    /// # let (public_key, secret_key) = IntegerElGamal::generate_keys(&BitsOfSecurity::Other {pk_bits: 160}, &mut rng);
+    /// # let ciphertext = IntegerElGamal::encrypt(&Integer::from(5), &public_key, &mut rng);
     /// let rich_ciphertext = ciphertext.enrich(&public_key);
-    /// println!("The decrypted message is {}", el_gamal.decrypt(&rich_ciphertext, &secret_key));
+    /// println!("The decrypted message is {}", IntegerElGamal::decrypt(&rich_ciphertext, &secret_key));
     /// // Prints: "The decrypted message is 5".
     /// ```
     fn decrypt(
-        &self,
         rich_ciphertext: &RichCiphertext<Self::Ciphertext, Self::PublicKey>,
         secret_key: &Self::SecretKey,
     ) -> Self::Plaintext {
@@ -208,9 +180,9 @@ impl<'pk> RichCiphertext<'pk, IntegerElGamalCiphertext, IntegerElGamalPublicKey>
 #[cfg(test)]
 mod tests {
     use crate::cryptosystems::integer_el_gamal::IntegerElGamal;
+    use crate::cryptosystems::AsymmetricCryptosystem;
     use crate::randomness::SecureRng;
-    use crate::AsymmetricCryptosystem;
-    use crate::Enrichable;
+    use crate::{BitsOfSecurity, Enrichable};
     use rand_core::OsRng;
     use rug::Integer;
 
@@ -218,14 +190,14 @@ mod tests {
     fn test_encrypt_decrypt_generator() {
         let mut rng = SecureRng::new(OsRng);
 
-        let el_gamal = IntegerElGamal::new(512, &mut rng);
-        let (pk, sk) = el_gamal.generate_keys(&mut rng);
+        let (pk, sk) =
+            IntegerElGamal::generate_keys(&BitsOfSecurity::Other { pk_bits: 160 }, &mut rng);
 
-        let ciphertext = el_gamal.encrypt(&Integer::from(19), &pk, &mut rng);
+        let ciphertext = IntegerElGamal::encrypt(&Integer::from(19), &pk, &mut rng);
 
         assert_eq!(
             Integer::from(19),
-            el_gamal.decrypt(&ciphertext.enrich(&pk), &sk)
+            IntegerElGamal::decrypt(&ciphertext.enrich(&pk), &sk)
         );
     }
 
@@ -233,32 +205,31 @@ mod tests {
     fn test_homomorphic_mul() {
         let mut rng = SecureRng::new(OsRng);
 
-        let el_gamal = IntegerElGamal::new(512, &mut rng);
-        let (pk, sk) = el_gamal.generate_keys(&mut rng);
+        let (pk, sk) =
+            IntegerElGamal::generate_keys(&BitsOfSecurity::Other { pk_bits: 160 }, &mut rng);
 
-        let ciphertext = el_gamal
-            .encrypt(&Integer::from(7), &pk, &mut rng)
-            .enrich(&pk);
+        let ciphertext = IntegerElGamal::encrypt(&Integer::from(7), &pk, &mut rng).enrich(&pk);
         let ciphertext_twice = &ciphertext * &ciphertext;
 
-        assert_eq!(Integer::from(49), el_gamal.decrypt(&ciphertext_twice, &sk));
+        assert_eq!(
+            Integer::from(49),
+            IntegerElGamal::decrypt(&ciphertext_twice, &sk)
+        );
     }
 
     #[test]
     fn test_homomorphic_scalar_pow() {
         let mut rng = SecureRng::new(OsRng);
 
-        let el_gamal = IntegerElGamal::new(512, &mut rng);
-        let (pk, sk) = el_gamal.generate_keys(&mut rng);
+        let (pk, sk) =
+            IntegerElGamal::generate_keys(&BitsOfSecurity::Other { pk_bits: 160 }, &mut rng);
 
-        let ciphertext = el_gamal
-            .encrypt(&Integer::from(9), &pk, &mut rng)
-            .enrich(&pk);
+        let ciphertext = IntegerElGamal::encrypt(&Integer::from(9), &pk, &mut rng).enrich(&pk);
         let ciphertext_twice = ciphertext.pow(&Integer::from(4));
 
         assert_eq!(
             Integer::from(6561),
-            el_gamal.decrypt(&ciphertext_twice, &sk)
+            IntegerElGamal::decrypt(&ciphertext_twice, &sk)
         );
     }
 }
